# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .amount_input_param import AmountInputParam

__all__ = ["FlatPriceInputParam"]


class FlatPriceInputParam(TypedDict, total=False):
    amount: Required[AmountInputParam]

    price_type: Literal["flat"]
